/*
 Copyright 2015 Nicolas Melot

 This file is part of Crown.

 Crown is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Crown is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Crown. If not, see <http://www.gnu.org/licenses/>.

*/


#include <pelib/Algebra.hpp>

#ifndef ALLOCATION_HPP
#define ALLOCATION_HPP

/** Computes task allocation that minimize the execution time of individual tasks **/
pelib::Algebra
allocation_fastest(const pelib::Algebra &input);

/** Computes a random task allocation **/
pelib::Algebra
allocation_random(const pelib::Algebra &input);

/** Compute a task allocation constrained by a minimal efficiency for tasks **/
pelib::Algebra
allocation_gemin(const pelib::Algebra &schedule, float gemin);

/** Compute an allocation that maximizes speedup of each tasks individually, given a minimum efficiency constraint per task in vector temin, or scalar gemin if temin doesn't exists, or 0.5 if neither temin or gemin is in input Algebra collection. **/
pelib::Algebra
allocation_temin(const pelib::Algebra &schedule);

/** Return the time in number of instructions as given by a time complexity analysis to run the fastest allocation algorithm with the given input problem. **/
float
allocation_fastest_complexity(const pelib::Algebra &schedule);

#endif
